-------------------------------------------------------------------------
--
--  timing_gen_640x400.vhd
--
--	Author 	: Rene Bllhoff
--  Create 	: Mai. 2006
--
--  Purpose : 
--
--	Erzeugt H-Sync/V-Sync/Pixel-Off und Adress-Signale fr VGA
--  Synchronisation mit SRAM_ACCESS-Komponente noch nicht ganz sauber
--  (wird nur einmal gesyncht, mu noch berarbeitet werden)
--
--
--  (c)2005-2006 by Rene Bllhoff
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity timing_gen640x400 is
    Port ( clkvga 			: in 	std_logic;
           res 					: in 	std_logic;

					 sync_tg			: in std_logic;

           addr_out		: out  std_logic_vector (19 downto 0);  -- addresse auf zeichen im text-ram

           hs_out     : out std_logic; 												-- horizontal sync
           vs_out     : out std_logic; 												-- vertikal sync
           hpos_out 	: out std_logic_vector ( 9 downto 0); 	-- akt. spalte
           vpos_out 	: out std_logic_vector ( 9 downto 0);  	-- akt. zeile
					 pixoff_out : out  std_logic												-- pixel aus (alles auf 0)
					);
end timing_gen640x400;

architecture Behavioral of timing_gen640x400 is

	type      output_mode is (pixel_output, back_porch, sync, front_porch);


	constant  h_pixel_length_sl : std_logic_vector (11 downto 0) := X"280";

	constant  h_pixel_length  	: integer := 640;
	constant  h_front_porch   	: integer := 16;
	constant  h_sync_length   	: integer := 96;
	constant  h_back_porch    	: integer := 48;

  constant  v_pixel_length  	: integer := 400;
  constant  v_front_porch   	: integer := 18;
	constant  v_sync_length   	: integer := 2;
	constant  v_back_porch    	: integer := 22;

	signal h_mode, h_mode_z1  	: output_mode;
	signal v_mode, v_mode_z1  	: output_mode;

	signal h_pos        				: std_logic_vector ( 9 downto 0);
	signal v_pos								: std_logic_vector ( 9 downto 0);

	signal hs_z1, vs_z1 				: std_logic;

	signal wait_sync						: std_logic;
	signal count								: std_logic_vector (2 downto 0);

begin

	process (clkvga, res)
	  variable h_count : integer range 0 to 1023;
		variable v_count : integer range 0 to 511;
	begin
		if res = '1' then
		  h_count 			:= 0;
			v_count 			:= 0;

			v_mode 				<= pixel_output;
			h_mode 				<= pixel_output;

			v_mode_z1 		<= pixel_output;
			h_mode_z1 		<= pixel_output;

			h_pos 				<= (others => '0');
			v_pos 				<= (others => '0'); 
			
			addr_out   	 	<= (others => '0'); 

			pixoff_out 		<= '1';
  		vs_z1 				<= '0';
	    hs_z1 				<= '0';

			wait_sync 		<= '1';
			count					<= "000";


		elsif clkvga'event and clkvga = '1'then


			if wait_sync = '1' then

				if count = "000" then
					if sync_tg = '1' then
					  count <= "001";
					end if;
				elsif count = "101" then
				  wait_sync <= '0';
				else
				  count <= count + 1;
				end if;

			else

				h_pos 		<= conv_std_logic_vector (h_count, 10);
				v_pos 		<= conv_std_logic_vector (v_count, 10);

				hpos_out 	<= h_pos;
				vpos_out 	<= v_pos;

				h_mode 		<= h_mode_z1;
				v_mode 		<= v_mode_z1;
      
				h_count 	:= h_count + 1;

				vs_out 		<= vs_z1;
				hs_out 		<= hs_z1;

				case h_mode_z1 is
					when pixel_output	=> if h_count = h_pixel_length then
																 h_count := 0;
																 h_mode_z1  <= front_porch;
															 end if;

					when front_porch	=> if h_count = h_front_porch then
																 h_count := 0;
																 h_mode_z1  <= sync;
															 end if;	

					when sync      		=> if h_count = h_sync_length then
																 h_count := 0;
																 h_mode_z1  <= back_porch;
															 end if;	

					when back_porch		=> if h_count = h_back_porch then	
																 h_count := 0;
																 h_mode_z1  <= pixel_output;
																 v_count := v_count + 1;	
															 end if;	
				end case;

				case v_mode_z1 is
					when pixel_output	=> if v_count = v_pixel_length then
																 v_count    := 0;
																 v_mode_z1     <= back_porch;
															 end if;	 	

					when back_porch		=> if v_count = v_back_porch then
																 v_count 		:= 0;
																 v_mode_z1  		<= sync;
															 end if;

					when sync      		=> if v_count = v_sync_length then
																 v_count 		:= 0;
																 v_mode_z1  		<= front_porch;
															 end if;

					when front_porch	=> if v_count = v_front_porch then
																 v_count    := 0;
																 v_mode_z1     <= pixel_output;
															 end if;
				end case;



				if v_mode = pixel_output and h_mode = pixel_output then
					pixoff_out   	<= '0';
					addr_out    	<= h_pos (9 downto 0) + (h_pixel_length_sl (9 downto 0)) * (v_pos (9 downto 0));
				else
					pixoff_out 		<= '1';
					addr_out 			<= (others => '0');
				end if;

				case v_mode is
	        when sync    => vs_z1 <= '0';
				  when others  => vs_z1 <= '1';
				end case;

				case h_mode is
	        when sync    => hs_z1 <= '0';
				  when others  => hs_z1 <= '1';
				end case;
			
			end if;


			


		end if;
	end process;

	
end Behavioral;
